/*
 * TrueMVC JavaScript with Objs - www.tekool.net
 * TrueMVC JavaScript may be freely distributed under the MIT license.
 * For all details and documentation: https://github.com/tekool/truemvc-javascript
 * Inspired from PureMVC - Copyright(c) 2006-2012 Futurescale, Inc., Some rights reserved.
 */
new function()
{
	/**
	 * @class truemvc.observer.Observer
	 *
	 * @classDescription
	 * The base <code>Observer</code> class.
	 *
	 * <P>
	 * Offers a way for to be registered as an interested object with a method that should be called when a particular
	 * <code>Notification</code> is broadcasted by the subject.
	 *
	 * <P>
	 * If all object-oriented languages had interfaces it would have been preferable to implement an Observer interface
	 * on observers object. But here we can safely use an <code>Observer</code> class as we can control when observer
	 * and observable actors are removed from each application.
	 *
	 * @see truemvc.mediator.Mediator
	 * @see truemvc.mvc.Controller
	 *
	 * @constructor
	 */
	Objs
	(
		"truemvc.observer.Observer",
		{
			/**
			 * @abstract
			 * Handle <code>Notification</code>s emitted by the subject.
			 *
			 * <P>
			 * This is the Observer update method.
			 *
			 * @param {Notification} notification
			 * 		The notification instance to be handled.
			 */
			handleNotification: function( notification )
			{
				throw Error("Observer.handleNotification method is abstract and must be overridden.");
			}
		}
	);
}