/*
 * Inspired from PureMVC - Copyright(c) 2006-2012 Futurescale, Inc., Some rights reserved.
 * TrueMVC JavaScript may be freely distributed under the MIT license.
 * For all details and documentation: https://github.com/tekool/truemvc-javascript
 */

/**
 * Test the TrueMVC Command class.
 *
 * @see truemvc.command.CommandTestVO CommandTestVO
 * @see truemvc.command.CommandTestCommand CommandTestCommand
 */
var CommandTest = new YUITest.TestCase
(
	{
	    /**
	     * The name of the test case - if not provided, one is automatically generated by the YUITest framework.
	     * 
	     * @type {String}
	     * @private
	     */
	    name: "TrueMVC Command class Tests",
	
	    /**
	     * Sets up data that is needed by each test.
	     */
	    setUp: function()
		{
	    },
	    
	    /**
	     * Cleans up everything that was created by setUp().
	     */
	    tearDown: function()
		{
	    },


		/**
		 * Tests if constructing the <code>Command</code> also allow to get and set a reference to <code>View</code>.
		 */
		testViewGetterSetters: function()
		{
			var Command = Objs("truemvc.command.Command");
			var View = Objs("truemvc.mvc.View");

			// Create a new subclass of Notifier and verify that its facade has well been set.
			var command/*Command*/ = new Command();

			//View is defaulted to strict null
			YUITest.Assert.isNull
			(
				command.getView(),
				"Expecting command.getView() === null"
			);

			var view = new View();
			command.setView(view);

			// test assertions
			YUITest.Assert.areEqual
			(
				command.getView(),
				view,
				"Expecting command.getView() === view"
			);
		},

		/**
		 * Tests if constructing the <code>Command</code> also allow to get and set a reference to <code>Model</code>.
		 */
		testModelGetterSetters: function()
		{
			var Command = Objs("truemvc.command.Command");
			var Model = Objs("truemvc.mvc.Model");

			// Create a new subclass of Notifier and verify that its facade
			// has well been set.
			var command/*Command*/ = new Command();

			//View is defaulted to strict null
			YUITest.Assert.isNull
			(
				command.getModel(),
				"Expecting command.getModel() === null"
			);

			var model = new Model();
			command.setModel(model);

			// test assertions
			YUITest.Assert.areEqual
			(
				command.getModel(),
				model,
				"Expecting command.getModel() === model"
			);
		},
	 		
		/**
		 * Tests the <code>execute</code> method of a <code>Command</code>.
		 * 
		 * <P>
		 * This test creates a new <code>Notification</code>, adding a <code>CommandTestVO</code> as the body. It then
		 * creates a <code>CommandTestCommand</code> and invokes its <code>execute</code> method, passing in the
		 * notification.
		 * 
		 * <P>
		 * Success is determined by evaluating a property on the object that was passed on the Notification body, which
		 * will be modified by the Command.
		 * 
		 */
		testCommandExecute: function()
		{
   			var Notification = Objs("truemvc.observer.Notification");

			// Create the VO
			var vo/*CommandTestVO*/ = new CommandTestVO(5);
			
			// Create the Notification
			var notification/*Notification*/ = new Notification( 'CommandTestNote', vo );
			
			// Create the Command
			var command/*CommandTestCommand*/ = new CommandTestCommand();
			
			// Execute the Command
			command.execute(notification);
			
			// test assertions
			YUITest.Assert.areEqual
			(
				10,
				vo.result,
				"Expecting vo.result == 10"
			);
		}
	}
);