/*
 * Inspired from PureMVC - Copyright(c) 2006-2012 Futurescale, Inc., Some rights reserved.
 * TrueMVC JavaScript may be freely distributed under the MIT license.
 * For all details and documentation: https://github.com/tekool/truemvc-javascript
 */

/**
 * Test the TrueMVC MacroCommmand class.
 *
 * @see truemvc.command.MacroCommandTestVO MacroCommandTestVO
 * @see truemvc.command.MacroCommandTestCommand MacroCommandTestCommand
 */
var MacroCommandTest = new YUITest.TestCase
(
	{
		/**
		 * The name of the test case - if not provided, one is automatically
		 * generated by the YUITest framework.
		 * 
		 * @type {String}
		 * @private
		 */
		name: "TrueMVC MacroCommmand class tests",
	
		/**
		 * Sets up data that is needed by each test.
		 */
		setUp: function()
		{
		},
	    
		/**
		 * Cleans up everything that was created by setUp().
		 */
		tearDown: function()
		{
		},

		/**
		 * Tests if constructing the <code>MacroCommand</code> also allow to get and set a reference to
		 * <code>View</code>.
		 */
		testViewGetterSetters: function()
		{
			var MacroCommand = Objs("truemvc.command.MacroCommand");
			var View = Objs("truemvc.mvc.View");

			// Create a new subclass of Notifier and verify that its facade
			// has well been set.
			var macroCommand/*MacroCommand*/ = new MacroCommand();

			//View is defaulted to strict null
			YUITest.Assert.isNull
			(
				macroCommand.getView(),
				"Expecting macroCommand.getView() === null"
			);

			var view = new View();
			macroCommand.setView(view);

			// test assertions
			YUITest.Assert.areEqual
			(
				macroCommand.getView(),
				view,
				"Expecting macroCommand.getView() === view"
			);
		},

		/**
		 * Tests if constructing the <code>MacroCommand</code> also allow to get and set a reference to
		 * <code>Model</code>.
		 */
		testModelGetterSetters: function()
		{
			var MacroCommand = Objs("truemvc.command.MacroCommand");
			var Model = Objs("truemvc.mvc.Model");

			// Create a new subclass of Notifier and verify that its facade
			// has well been set.
			var macroCommand/*MacroCommand*/ = new MacroCommand();

			//View is defaulted to strict null
			YUITest.Assert.isNull
			(
				macroCommand.getModel(),
				"Expecting macroCommand.getModel() === null"
			);

			var model = new Model();
			macroCommand.setModel(model);

			// test assertions
			YUITest.Assert.areEqual
			(
				macroCommand.getModel(),
				model,
				"Expecting macroCommand.getModel() === model"
			);
		},

		/**
		 * Tests operation of a <code>MacroCommand</code>.
		 * 
		 * <P>
		 * This test creates a new <code>Notification</code>, adding a 
		 * <code>MacroCommandTestVO</code> as the body. 
		 * It then creates a <code>MacroCommandTestCommand</code> and invokes
		 * its <code>execute</code> method, passing in the 
		 * <code>Notification</code>.
		 * 
		 * <P>
		 * The <code>MacroCommandTestCommand</code> has defined an
		 * <code>initMacroCommand</code> method, which is 
		 * called automatically by its constructor. In this method
		 * the <code>MacroCommandTestCommand</code> adds 2 SubCommands
		 * to itself, <code>MacroCommandTestSub1Command</code> and
		 * <code>MacroCommandTestSub2Command</code>.
		 * 
		 * <P>
		 * The <code>MacroCommandTestVO</code> has 2 result properties,
		 * one is set by <code>MacroCommandTestSub1Command</code> by
		 * multiplying the input property by 2, and the other is set
		 * by <code>MacroCommandTestSub2Command</code> by multiplying
		 * the input property by itself. 
		 * 
		 * <P>
		 * Success is determined by evaluating the 2 result properties
		 * on the <code>MacroCommandTestVO</code> that was passed to 
		 * the <code>MacroCommandTestCommand</code> on the Notification 
		 * body.
		 * 
		 */
		testMacroCommandExecute: function()
		{
   			var Notification = Objs("truemvc.observer.Notification");

			// Create the VO
			var vo/*MacroCommandTestVO*/ = new MacroCommandTestVO(5);
			
			// Create the Notification (notification)
			var notification/*Notification*/ = new Notification( 'MacroCommandTest', vo );
	
			// Create the MacroCommand  			
			var command/*MacroCommandTestCommand*/ = new MacroCommandTestCommand();
		
			// Execute the MacroCommand
			command.execute(notification);
		
			// test assertions
			YUITest.Assert.areEqual
			(
				10,
				vo.result1,
				"Expecting vo.result1 == 10"
			);
			
			YUITest.Assert.areEqual
			(
				25,
				vo.result2,
				"Expecting vo.result2 == 25"
			);
		}
	}
);