/*
 * Inspired from PureMVC - Copyright(c) 2006-2012 Futurescale, Inc., Some rights reserved.
 * TrueMVC JavaScript may be freely distributed under the MIT license.
 * For all details and documentation: https://github.com/tekool/truemvc-javascript
 */

/**
 * Test the TrueMVC Facade class.
 *
 * @see truemvc.facade.FacadeTestVO
 * @see truemvc.facade.FacadeTestCommand
 */
var FacadeTest = new YUITest.TestCase
(
	{

       /**
        * The name of the test case - if not provided, one is automatically
		* generated by the YUITest framework.
		*
		* @type {String}
		* @private
		*/
        name: "TrueMVC Facade class tests",

        /**
         * Sets up data that is needed by each test.
         */
        setUp: function()
		{
        },
        
        /**
         * Cleans up everything that was created by setUp().
         */
        tearDown: function()
		{
        },

		//TODO test for model, view, controller calls to initialize in disorder

		/**
		 * Tests the Facade Constructor.
		 *
		 * <P>
		 * This test constructs and initializes a Facade instance and check to see if it correctly also set and
		 * initialize its <code>Model</code>, <code>View</code> and <code>Controller</code>.
		 */
		testConstructor: function()
		{
			var Facade = Objs("truemvc.facade.Facade");
			var Model = Objs("truemvc.mvc.Model");
			var View = Objs("truemvc.mvc.View");
			var Controller = Objs("truemvc.mvc.Controller");

			var facade/*Facade*/ = new Facade();

			var model/*Model*/ = facade.getModel();
			var view/*View*/ = facade.getView();
			var controller/*Controller*/ = facade.getController();

			//Assert that the model, view and controller are automatically created when calling the default constructor.
  			YUITest.Assert.isNotNull( model, "Expecting facade.getModel() not to be null" );
   			YUITest.Assert.isNotNull( view, "Expecting facade.getView() not to be null" );
   			YUITest.Assert.isNotNull( controller, "Expecting facade.getController() not to be null" );

			//Assert that the model, view and controller are automatically created when calling the default constructor.
  			YUITest.Assert.isInstanceOf( Model, model, "Expecting facade.getModel() to be an instance of Model" );
   			YUITest.Assert.isInstanceOf( View, view, "Expecting facade.getView() to be an instance of View" );
   			YUITest.Assert.isInstanceOf( Controller, controller, "Expecting facade.getController() to be an instance of Controller" );

			//Assert that the model, view and controller are automatically initialized when calling the default constructor.
  			YUITest.Assert.areSame( model, view.getModel(), "Expecting view to be initialized with model" );
  			YUITest.Assert.areSame( model, controller.getModel(), "Expecting controller to be initialized with model" );
  			YUITest.Assert.areSame( view, controller.getView(), "Expecting controller to be initialized with view" );
		},

		/**
		 * Tests the Facade Constructor.
		 *
		 * <P>
		 * This test constructs and initializes a Facade subclass instance and check to see if it correctly also set and
		 * initialize its <code>Model</code>, <code>View</code> and <code>Controller</code> in the right order.
		 */
		testConstructorAutomaticInitialization: function()
		{
			var FacadeTestSub2 = Objs("truemvc.facade.FacadeTestSub2");

			var facade/*FacadeTestSub2*/ = new FacadeTestSub2();

			var model/*Model*/ = facade.getModel();
			var view/*View*/ = facade.getView();
			var controller/*Controller*/ = facade.getController();

			//Assert that the model, view and controller are automatically created and initialized in the right order.
  			YUITest.Assert.areEqual( facade.facadeInitialized, 1, "Expecting 1" );

  			YUITest.Assert.areEqual( facade.modelSet, 2, "Expecting 2" );
  			YUITest.Assert.areEqual( facade.viewSet, 3, "Expecting 3" );
  			YUITest.Assert.areEqual( facade.controllerSet, 4, "Expecting 4" );

  			YUITest.Assert.areEqual( facade.modelInitialized, 5, "Expecting 5" );
  			YUITest.Assert.areEqual( facade.viewInitialized, 6, "Expecting 6" );
  			YUITest.Assert.areEqual( facade.controllerInitialized, 7, "Expecting 7" );
		},

		/**
		 * Tests the Facade model getter/setter.
		 */
		testGetSetModel: function()
		{
			var Model = Objs("truemvc.mvc.Model");
			var FacadeTestSub = Objs("truemvc.facade.FacadeTestSub");

			var facade/*FacadeTestSub*/ = new FacadeTestSub();

			//Assert that the model is null when contructing a Facade with not doing any automatic initialization.
  			YUITest.Assert.isNull( facade.getModel(), "Model expected to be null" );

			//Assert that the model getter/setter behave as expected.
			var model/*Model*/ = new Model();
			facade.setModel( model );

  			YUITest.Assert.areSame( facade.getModel(), model, "obtained model expected to be the same as the one set" );
		},

		/**
		 * Tests the Facade view getter/setter.
		 */
		testGetSetView: function()
		{
			var View = Objs("truemvc.mvc.View");
			var FacadeTestSub = Objs("truemvc.facade.FacadeTestSub");

			var facade/*FacadeTestSub*/ = new FacadeTestSub();

			//Assert that the view is null when contructing a Facade with not doing any automatic initialization.
  			YUITest.Assert.isNull( facade.getView(), "View expected to be null" );

			//Assert that the view getter/setter behave as expected.
			var view/*View*/ = new View();
			facade.setView( view );

  			YUITest.Assert.areSame( facade.getView(), view, "obtained view expected to be the same as the one set" );
		},

		/**
		 * Tests the Facade model getter/setter.
		 */
		testGetSetController: function()
		{
			var Controller = Objs("truemvc.mvc.Controller");
			var FacadeTestSub = Objs("truemvc.facade.FacadeTestSub");

			var facade/*FacadeTestSub*/ = new FacadeTestSub();

			//Assert that the controller is null when contructing a Facade with not doing any automatic initialization.
  			YUITest.Assert.isNull( facade.getController(), "Controller expected to be null" );

			//Assert that the controller getter/setter behave as expected.
			var controller/*Controller*/ = new Controller();
			facade.setController( controller );

  			YUITest.Assert.areSame( facade.getController(), controller, "obtained controller expected to be the same as the one set" );
		},

		/**
		 * Tests the InitializeFacade Method.
		 *
		 * <P>
		 * This test constructs and initializes a Facade instance and check to see if it correctly also set and
		 * initialize its <code>Model</code>, <code>View</code> and <code>Controller</code>.
		 *
		 * <P>
		 *     Rem: Same as the default controller test but using a subclass of <code>Facade</code>.
		 */
		testInitializeFacade: function()
		{
			var FacadeTestSub = Objs("truemvc.facade.FacadeTestSub");
			var Model = Objs("truemvc.mvc.Model");
			var View = Objs("truemvc.mvc.View");
			var Controller = Objs("truemvc.mvc.Controller");

			var facade/*FacadeTestSub*/ = new FacadeTestSub();

			//Assert that the model, view and controller are not automatically created when constructing the subclass.
  			YUITest.Assert.isNull( facade.getModel(), "Expecting facade.getModel() to be null" );
   			YUITest.Assert.isNull( facade.getView(), "Expecting facade.getView() to be null" );
   			YUITest.Assert.isNull( facade.getController(), "Expecting facade.getController() to be null" );

			facade.initializeFacade();

			var model/*Model*/ = facade.getModel();
			var view/*View*/ = facade.getView();
			var controller/*Controller*/ = facade.getController();

			//Assert that the model, view and controller are automatically created when calling the default constructor.
  			YUITest.Assert.isNotNull( model, "Expecting facade.getModel() not to be null" );
   			YUITest.Assert.isNotNull( view, "Expecting facade.getView() not to be null" );
   			YUITest.Assert.isNotNull( controller, "Expecting facade.getController() not to be null" );

			//Assert that the model, view and controller are automatically created when calling the default constructor.
  			YUITest.Assert.isInstanceOf( Model, model, "Expecting facade.getModel() to be an instance of Model" );
   			YUITest.Assert.isInstanceOf( View, view, "Expecting facade.getView() to be an instance of View" );
   			YUITest.Assert.isInstanceOf( Controller, controller, "Expecting facade.getController() to be an instance of Controller" );

			//Assert that the model, view and controller are automatically initialized when calling the default constructor.
  			YUITest.Assert.areSame( model, view.getModel(), "Expecting view to be initialized with model" );
  			YUITest.Assert.areSame( model, controller.getModel(), "Expecting controller to be initialized with model" );
  			YUITest.Assert.areSame( view, controller.getView(), "Expecting controller to be initialized with view" );
		},

		//TODO We need a dedicated test for initializeController, initializeModel, initializeView  (9 call order possiblities)
		//TODO We need a dedicated test for removeAllCommands
		//TODO We need a dedicated test for removeAllProxies
		//TODO We need a dedicated test for removeAllMediators

  		/**
  		 * Tests Command registration and execution via the Facade.
  		 * 
  		 * <P>
  		 * This test gets the Singleton Facade instance 
  		 * and registers the FacadeTestCommand class 
  		 * to handle 'FacadeTest' Notifications.
  		 * 
  		 * <P>
  		 * It then sends a notification using the Facade. 
  		 * Success is determined by evaluating 
  		 * a property on an object placed in the body of
  		 * the Notification, which will be modified by the Command.
  		 * 
  		 */
  		testRegisterCommandAndSendNotification: function()
		{
			var Facade = Objs("truemvc.facade.Facade");

   			// Create the Facade, register the FacadeTestCommand to
   			// handle 'FacadeTest' notifications
   			var facade/*Facade*/ = new Facade();
   			facade.registerCommand( 'FacadeTestNote', "truemvc.facade.FacadeTestCommand" );

			// Send notification. The Command associated with the event
			// (FacadeTestCommand) will be invoked, and will multiply 
			// the vo.input value by 2 and set the result on vo.result
   			var vo/*FacadeTestVO*/ = new FacadeTestVO( 32 );
   			facade.sendNotification( 'FacadeTestNote', vo );
   			
   			// test assertions 
   			YUITest.Assert.areEqual
			(
				64,
				vo.result,
				"Expecting vo.result == 64"
			);
   		},

  		/**
  		 * Tests Command removal via the Facade.
  		 * 
  		 * <P>
  		 * This test gets the Singleton Facade instance 
  		 * and registers the FacadeTestCommand class 
  		 * to handle 'FacadeTest' Notifcations. Then it removes the command.
  		 * 
  		 * <P>
  		 * It then sends a Notification using the Facade. 
  		 * Success is determined by evaluating a property on an object placed
  		 * in the body of the Notification, which will NOT be modified by the
  		 * Command.
  		 * 
  		 */
  		testRegisterAndRemoveCommandAndSendNotification: function()
		{
			var Facade = Objs("truemvc.facade.Facade");

   			// Create the Facade, register the FacadeTestCommand to
   			// handle 'FacadeTest' events
   			var facade/*Facade*/ = new Facade();
   			facade.registerCommand( 'FacadeTestNote', "truemvc.facade.FacadeTestCommand" );
   			facade.removeCommand( 'FacadeTestNote' );

			// Send notification. The Command associated with the event
			// (FacadeTestCommand) will NOT be invoked, and will NOT multiply 
			// the vo.input value by 2 
   			var vo/*Object*/ = new FacadeTestVO( 32 );
   			facade.sendNotification( 'FacadeTestNote', vo );
   			
   			// test assertions 
   			YUITest.Assert.areNotEqual
			(
				64,
				vo.result,
				"Expecting vo.result != 64"
			);
   		},

  		/**
  		 * Tests the registering and retrieving Model proxies via the Facade.
  		 * 
  		 * <P>
  		 * Tests <code>registerProxy</code> and <code>retrieveProxy</code> in the same test.
  		 * These methods cannot currently be tested separately
  		 * in any meaningful way other than to show that the
  		 * methods do not throw exception when called.
  		 */
  		testRegisterAndRetrieveProxy: function()
		{
			var Facade = Objs("truemvc.facade.Facade");
	   		var Proxy = Objs("truemvc.proxy.Proxy");

   			// register a proxy and retrieve it.
   			var facade/*Facade*/ = new Facade();
			facade.registerProxy( new Proxy( 'colors', ['red', 'green', 'blue'] ) );
			
			var proxy/*Proxy*/ = facade.retrieveProxy( 'colors' );

   			YUITest.Assert.isInstanceOf
			(
				Proxy,
				proxy,
				"Expecting proxy is Proxy"
			);

			// retrieve data from proxy
			var data/*Array*/ = proxy.getData();
			
			// test assertions
   			YUITest.Assert.isNotUndefined
			(
				data,
				"Expecting data not null"
			);
			
   			YUITest.Assert.isArray
			(
				data,
				"Expecting data is Array"
			);
			
   			YUITest.Assert.areEqual
			(
				3,
				data.length,
				"Expecting data.length == 3"
			);

   			YUITest.Assert.areEqual
			(
				'red',
				data[0],
				"Expecting data[0] == 'red'"
			);

   			YUITest.Assert.areEqual
			(
				'green',
				data[1],
				"Expecting data[1] == 'green'"
			);
			
   			YUITest.Assert.areEqual
			(
				'blue',
				data[2],
				"Expecting data[2] == 'blue'"
			);
   		},
  		
  		/**
  		 * Tests the removing Proxies via the Facade.
  		 */
  		testRegisterAndRemoveProxy: function()
		{
			var Facade = Objs("truemvc.facade.Facade");
    		var Proxy = Objs("truemvc.proxy.Proxy");

   			// register a proxy, remove it, then try to retrieve it
   			var facade/*Facade*/ = new Facade();
   			var proxy/*Proxy*/ = new Proxy( 'sizes', ['7', '13', '21'] );
			facade.registerProxy( proxy );
			
			// remove the proxy
			var removedProxy/*Proxy*/ = facade.removeProxy('sizes');
			
			// test assertions

			// assert that we removed the appropriate proxy
   			YUITest.Assert.areEqual
			(
				'sizes',
   				removedProxy ? removedProxy.getProxyName() : null,
				"Expecting removedProxy.getProxyName() == 'sizes'"
			);
			
			// make sure we can no longer retrieve the proxy from the model
			proxy = facade.retrieveProxy( 'sizes' );
			
			// assert that the proxy is no longer retrievable
   			YUITest.Assert.isNull( proxy, "Expecting proxy === null" );
   		},

   		
  		/**
  		 * Tests registering, retrieving and removing Mediators via the Facade.
  		 */
  		testRegisterRetrieveAndRemoveMediator: function()
		{
			var Facade = Objs("truemvc.facade.Facade");
    		var Mediator = Objs("truemvc.mediator.Mediator");

   			// register a mediator, remove it, then try to retrieve it
   			var facade/*Facade*/ = new Facade();
			facade.registerMediator( new Mediator( Mediator.NAME, new Object() ) );
			
			// retrieve the mediator
   			YUITest.Assert.isNotNull
			(
				facade.retrieveMediator( Mediator.NAME ),
				"Expecting facade.retrieveMediator( Mediator.NAME ) !== null"
			);

			// remove the mediator
			var removedMediator/*Mediator*/ = facade.removeMediator(Mediator.NAME);
			
			// assert that we have removed the appropriate mediator
   			YUITest.Assert.areEqual
			(
				Mediator.NAME,
				removedMediator ? removedMediator.getMediatorName() : null,
				"Expecting removedMediator.getMediatorName() == Mediator.NAME"
			);
				
			// assert that the mediator is no longer retrievable
   			YUITest.Assert.isNull
			( 
				facade.retrieveMediator( Mediator.NAME ),
				"Expecting facade.retrieveMediator( Mediator.NAME ) === null )"
			);		  			
   		},

  		/**
  		 * Tests the hasProxy Method
  		 */
  		testHasProxy: function()
		{
			var Facade = Objs("truemvc.facade.Facade");
    		var Proxy = Objs("truemvc.proxy.Proxy");

   			// register a Proxy
			var facade/*Facade*/ = new Facade();
			facade.registerProxy( new Proxy( 'hasProxyTest', [1,2,3] ) );
			
   			// assert that the model.hasProxy method returns true
   			// for that proxy name
   			YUITest.Assert.isTrue
			(
   				facade.hasProxy('hasProxyTest'),
				"Expecting facade.hasProxy('hasProxyTest') === true"
			);
   		},

  		/**
  		 * Tests the hasMediator Method
  		 */
  		testHasMediator: function()
		{
			var Facade = Objs("truemvc.facade.Facade");
     		var Mediator = Objs("truemvc.mediator.Mediator");

   			// register a Mediator
			var facade/*Facade*/ = new Facade();
			facade.registerMediator( new Mediator( 'facadeHasMediatorTest', new Object() ) );
			
   			// assert that the facade.hasMediator method returns true
   			// for that mediator name
   			YUITest.Assert.isTrue
			(
   				facade.hasMediator('facadeHasMediatorTest'),
				"Expecting facade.hasMediator('facadeHasMediatorTest') === true"
			);
   						
   			facade.removeMediator( 'facadeHasMediatorTest' );
   			
   			// assert that the facade.hasMediator method returns false
   			// for that mediator name
   			YUITest.Assert.isFalse
			(
   				facade.hasMediator('facadeHasMediatorTest'),
				"Expecting facade.hasMediator('facadeHasMediatorTest') === false"
			);
   		},

  		/**
  		 * Test hasCommand method.
  		 */
  		testHasCommand: function()
		{
			var Facade = Objs("truemvc.facade.Facade");

   			// register the ControllerTestCommand to handle 'hasCommandTest' notifications
   			var facade/*Facade*/ = new Facade();
   			facade.registerCommand( 'facadeHasCommandTest', "truemvc.facade.FacadeTestCommand" );
   			
   			// test that hasCommand returns true for hasCommandTest notifications 
   			YUITest.Assert.isTrue
			(
				facade.hasCommand('facadeHasCommandTest'),
				"Expecting facade.hasCommand('facadeHasCommandTest') === true"
			);
   			
   			// Remove the Command from the Controller
   			facade.removeCommand('facadeHasCommandTest');
			
   			// test that hasCommand returns false for hasCommandTest notifications 
   			YUITest.Assert.isFalse
			(
				facade.hasCommand('facadeHasCommandTest'),
				"Expecting facade.hasCommand('facadeHasCommandTest') === false"
			);
   		}
  	}
);