/*
 * Inspired from PureMVC - Copyright(c) 2006-2012 Futurescale, Inc., Some rights reserved.
 * TrueMVC JavaScript may be freely distributed under the MIT license.
 * For all details and documentation: https://github.com/tekool/truemvc-javascript
 */

/**
 * @classDescription
 * A Mediator class used by ViewTest.
 * 
 * @see truemvc.mvc.ViewTest ViewTest
 *
 * @extends truemvc.mediator.Mediator
 * 
 * @constructor
 */
var ViewTestMediator4 = Objs
(
	"truemvc.mvc.ViewTestMediator4",
	"truemvc.mediator.Mediator",
	{
		/**
		 * @constructs
		 * Initialize a <code>Mediator</code> subclass instance.
		 * 
		 * @param {Object} view
		 * 		The view component handled by this <code>Mediator</code>.
		 */
		initialize: function( view )
		{
			ViewTestMediator4.$super.initialize.call( this, ViewTestMediator4.NAME, view );
		},
				
		/**
		 * Standard getter to return the view handled by the
		 * <code>Mediator</code>.
		 * 
		 * @return {Object}
		 * 		The view handled by the <code>Mediator</code>.
		 * 
		 * @private
		 */
		getViewTest: function()
		{
			return this.viewComponent;
		},
			
		/**
		 * @override
		 */	
		onRegister: function()
		{
			this.getViewTest().onRegisterCalled = true;
		},
				
			
		/**
		 * @override
		 */	
		onRemove: function()
		{
			this.getViewTest().onRemoveCalled = true;
		}
	}
);

/**
 * The Mediator name.
 * 
 * @type {String}
 * @private
 */
ViewTestMediator4.NAME = 'ViewTestMediator4';