/*
 * Inspired from PureMVC - Copyright(c) 2006-2012 Futurescale, Inc., Some rights reserved.
 * TrueMVC JavaScript may be freely distributed under the MIT license.
 * For all details and documentation: https://github.com/tekool/truemvc-javascript
 */

/**
 * @classDescription
 * A Mediator class used by ViewTest.
 * 
 * @see truemvc.mvc.ViewTest ViewTest
 *
 * @extends truemvc.mediator.Mediator
 * 
 * @constructor
 */
var ViewTestMediator6 = Objs
(
	"truemvc.mvc.ViewTestMediator6",
	"truemvc.mediator.Mediator",
	{
		/**
		 * Standard getter to return the view handled by the
		 * <code>Mediator</code>.
		 * 
		 * @return {Object}
		 * 		The view handled by the <code>Mediator</code>.
		 * 
		 * @private
		 */
		getViewTest: function()
		{
			return this.viewComponent;
		},
	
		/**
		 * @override
		 *
		 * @return {Array}
		 * 		The list of notifications names in which is interested the
		 * 		<code>Mediator</code>.
		 */
		listNotificationInterests: function()
		{
			return [ ViewTest.NOTE6 ];
		},

		/**
		 * @override
		 *
		 * @param {Notification} notification
		 * 		The notification instance to be handled.
		 */
		handleNotification: function( notification )
		{
			//TODO We don't have to test the view here
			this.view.removeMediator(this.getMediatorName());
		},
	
		/**
		 * @override
		 */
		onRemove: function()
		{
			this.getViewTest().counter++;
		}
	}
);

/**
 * The Mediator name.
 * 
 * @type {String}
 * @private
 */
ViewTestMediator6.NAME = 'ViewTestMediator6';