/*
 * Inspired from PureMVC - Copyright(c) 2006-2012 Futurescale, Inc., Some rights reserved.
 * TrueMVC JavaScript may be freely distributed under the MIT license.
 * For all details and documentation: https://github.com/tekool/truemvc-javascript
 */

/**
 * Test the TrueMVC Notification class.
 * 
 * @see truemvc.observer.Notification
 */
var NotificationTest = new YUITest.TestCase
(
	{	
       /**
         * The name of the test case - if not provided, one is automatically
         * generated by the YUITest framework.
         * 
         * @type {String}
         * @private
         */
        name: "TrueMVC Notification class tests",

        /**
         * Sets up data that is needed by each test.
         */
        setUp: function()
		{
        },
        
        /**
         * Cleans up everything that was created by setUp().
         */
        tearDown: function()
		{
        },
        
  		/**
  		 * Tests setting and getting the name using Notification class accessor
  		 * methods.
  		 */
  		testNameAccessors: function()
		{
    		var Notification = Objs("truemvc.observer.Notification");

			// Create a new Notification and use accessors to set the notification name.
   			var notification/*Notification*/ = new Notification('TestNote');
  			
   			// test assertions
   			YUITest.Assert.areEqual
			(
				'TestNote',
				notification.getName(),
				"Expecting notification.getName() == 'TestNote'"
			);
   		},

  		/**
  		 * Tests setting and getting the body using Notification class accessor
  		 * methods.
  		 */
  		testBodyAccessors: function()
		{
    		var Notification = Objs("truemvc.observer.Notification");

			// Create a new Notification and use accessors to set the body
   			var notification/*Notification*/ = new Notification(null);
   			notification.setBody(5);
   			
   			// test assertions
   			YUITest.Assert.areSame
			(
				5,
				notification.getBody(),
				"Expecting notification.getBody() === 5"
			);
   		},

  		/**
  		 * Tests setting the name and body using the Notification class
  		 * Constructor.
  		 */
  		testConstructor: function()
		{
    		var Notification = Objs("truemvc.observer.Notification");

			// Create a new Notification using the Constructor to set the notification name and body
   			var notification/*Notification*/ = new Notification( 'TestNote', 5, 'TestNoteType' );
   			
   			// test assertions
   			YUITest.Assert.areEqual
			(
				"TestNote",
				notification.getName(),
				"Expecting notification.getName() == 'TestNote'"
			);
			
   			YUITest.Assert.areSame
			(
				5,
				notification.getBody(),
				"Expecting notification.getBody() === 5"
			);
			
   			YUITest.Assert.areEqual
			(
				"TestNoteType",
				notification.getType(),
				"Expecting notification.getType() == 'TestNoteType'"
			);
   		},

  		/**
  		 * Tests the toString method of the notification
  		 */
  		testToString: function()
		{
    		var Notification = Objs("truemvc.observer.Notification");

			// Create a new Notification and use accessors to set the notification name
   			var notification/*Notification*/ = new Notification( 'TestNote', [1,3,5], 'TestType' );
   			var ts/*String*/ = "Notification Name: TestNote\nBody:1,3,5\nType:TestType";
   			
   			// test assertions
   			YUITest.Assert.areEqual
			(
				ts,
				notification.toString(),
				"Expecting notification.testToString() == '" + ts + "'"
			);
   		}
  	}
);